"""
This plan class has four primitive goal-module-action triples,
and two compounds with a timer.  """

from planBaseClass import PlanBaseClass
from timerClass import TimerClass

class PlanClass (PlanBaseClass):
    NUMBER_GOALS = 6
    goalTurnRight = 0
    goalTurnLeft = 1
    goalStepForward = 2
    goalStepBackward = 3
    goalForwardAndRight = 4
    goalForwardAndLeft = 5

    NUMBER_MODULES = 8
    moduleTurnRight = 0
    moduleTurnLeft = 1
    moduleStepForward = 2
    moduleStepBackward = 3
    moduleForwardOfForwardRight = 4
    moduleRightOfForwardRight = 5
    moduleForwardOfForwardLeft = 6
    moduleLeftOfForwardLeft = 7

    NUMBER_FACTS = 1
    bumpFact = 0

    NUMBER_ACTIONS = 4
    actionRight = 0
    actionLeft = 1
    actionForward = 2
    actionBackward = 3

    def __init__(self, simName,sim,neal,spinnVersion,fsa):
        PlanBaseClass.__init__(self,simName,sim,neal,spinnVersion,fsa)
        self.simName = simName
        self.sim = sim
        self.neal = neal
        self.spinnVersion = spinnVersion
        self.fsa = fsa
        self.initializeBasePlanner()


    def makeTimers(self):
        self.forwardRightTimer = TimerClass(self.simName,self.sim,self.neal,
                                            self.spinnVersion,self.fsa,10)

        self.forwardLeftTimer = TimerClass(self.simName,self.sim,self.neal,
                                            self.spinnVersion,self.fsa,5)

    def connectCompoundPlanner(self):
        self.makeTimers()
        self.makePrimitive(self.goalTurnRight,self.moduleTurnRight,
                           self.actionRight)
        self.makePrimitive(self.goalTurnLeft,self.moduleTurnLeft,
                           self.actionLeft)
        self.makePrimitive(self.goalStepForward,self.moduleStepForward,
                           self.actionForward)
        self.makePrimitive(self.goalStepBackward,self.moduleStepBackward,
                           self.actionBackward)

        self.forwardRightTimer.makeStopTimerSynapses()
        self.makeCompound(self.goalForwardAndRight,
                          self.moduleForwardOfForwardRight,
                          self.moduleRightOfForwardRight,self.actionForward,
                          self.actionRight,self.forwardRightTimer)

        self.forwardLeftTimer.makeStopTimerSynapses()
        self.makeCompound(self.goalForwardAndLeft,
                          self.moduleForwardOfForwardLeft,
                          self.moduleLeftOfForwardLeft,self.actionForward,
                          self.actionLeft,self.forwardLeftTimer)
 
    def printResults(self):
        PlanBaseClass.printResults(self)
        self.forwardRightTimer.printResults("results/forwardRightTimer")
        self.forwardLeftTimer.printResults("results/forwardLeftTimer")

    def testStimulateFact(self,factNumber,spikeGenerator):
        self.fsa.turnOnStateFromSpikeSource(spikeGenerator,
                                            self.factCells, factNumber)

