""" 
This is just to build and test the spinnaker version of the neural language generator defined in LanguageGenerator.py.  That is, to build the
system and test the generation of one sentence.   

The language (i.e. the sentences the system can generate) are defined in chatbotLanguage.txt

To run the program enter the following into a shell: python testOneSentenceSpinn8.py sentenceNumber, for example
python testOneSentenceSpinn8.py 1.

sentenceNumber=1 should generate sentence "The red apple is in Room 1 1."
sentenceNumber=2 should generate sentence "The yellow banana is in Room 1 2."
sentenceNumber=3 should generate sentence "The diamond is oval."

Once the code has run, it should: 
1.  print the sentence generated by the system
2. generate a file OutputWords.txt containing all of the different words comprising the language (sentences) in file chatbotLanguage.txt
3. generate three pkl files, which represent the generation of the sentence in the results directory.

genSentence.pkl shows the neurons (CAs) that fired during the sentence generation. Each CA represents a sentence state, a sequence of states form a sentence.

genPhrase.pkl shows the neurons (phrase CAs) that fired during the sentence generation, each phrase CA corresponds to a different phrase.

genWords.pkl shows the word neurons that fired, each neuron corresponds to a different word.

see generatorBaseClass.py for more information about the sentence generation. 

"""

import pyNN.spiNNaker as sim
import sys

from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions
from LanguageGenerator import GeneratorClass

def init():
    sim.setup(timestep=neal.DELAY,
                    min_delay=neal.DELAY,
                    max_delay=neal.DELAY, debug=0)

#--main
args = sys.argv
numberArgs = args.__len__()
if (numberArgs == 2):
    sentenceToGenerate = int(args[1])# which sentence to test/output
else:
    #default
    sentenceToGenerate=3 # which sentence to test/output. 3 is the default

###MAIN
simName = "spinnaker"
spinnVersion = 8
LanguageFile="chatbotLanguage.txt"  #this is the file containing the chatbot's language (sentences).
OutputWordsFile="OutputWords.txt"  #where to write all of the different words comprising the language (sentences)
                                   #in the LanguageFile. 
neal = NealCoverFunctions(simName,sim,spinnVersion)
init()
fsa = FSAHelperFunctions(simName,sim,neal,spinnVersion)
generator = GeneratorClass(simName,sim,neal,spinnVersion,fsa,LanguageFile,OutputWordsFile)

startTime=100.0 ##when to start processing the sentence
generator.TestSentence(sentenceToGenerate,startTime) 

longestSentenceSize=generator.getLongestSentence() # determine the size of the longest sentence 
                                                   #in the given LanguageFile


##takes about 50ms to process each word in a sentence

SIM_LENGTH = 50.0*longestSentenceSize

neal.nealApplyProjections()
sim.run(SIM_LENGTH)   #process the sentence

generator.saveGenResults('results')  #save the pkl files

generator.printSentenceFromPkl() ##print the sentence generated by the network using the spiking results from genWords.pkl
			         #and the words contained in OutputWords.txt 

sim.end()
